# awk script to analyze 073.iobenchp results
#
#    Field         Contents
#
#      1           Number of files (input)
#      2           Number of users (input)
#      3           Record length (input)
#      4           Number of records (input)
#      5           Number of reads (input)
#      6           Number of writes (input)
#      7           CPU work parameter (input)
#      8           Number of cycles (input or output)
#      9           Cycles per second (output)
#     10           Actual time (output)
#     11           Flush time (output)
#     12           User CP seconds (output)
#     13           System CP seconds (output)
#     14           Total average response time (output)
#     15           Number of failed users (output)
#     16           Number of lock hits (output)
#
BEGIN { getline;print "USERS   RECLEN   CYCLES    MB   ACTUAL   FLUSH   USER    SYS   USER/SYS   TART" }
{	if ($2 != prev2) {
		users[n] = prev2
		cycles[n] = c;
		bytes[n] = b;
		actual[n]= ta;
		flush[n] = tf;
		n++;
		prev2 = $2;
		c = 0; ta = 0; tf = 0;
		}
c = c + $8; ta = ta + $10; tf = tf + $11; b = b + $8 * $3;
printf "%5d%9d%9d%6.1f%9d%8d%7.1f%7.1f%11.2f%7.1f\n", $2,$3,$8,($8 * $3)*1e-6,$10,$11,$12,$13,($12/$13),$14}


END {	users[n] = prev2
	cycles[n] = c;
	bytes[n] = b;
	actual[n] = ta;
	flush[n] = tf;

printf "Summary throughput/user follows\nUSERS   CYCLES    MB     TIME       CPS      MBS\n";
	for (i=1;i<=n;i++) {
		time = actual[i] + flush[i];
		mb = bytes[i] * 1e-6;
		cps = cycles[i] / time;
		printf  "%5d %8d%6.1f %8d  %8.1f %8.1f\n",users[i],cycles[i],mb,time,cps,mb/time
		}
}

