/* ioflush.c
/* Copyright (c) 1990, Prime Computer, Inc., Natick, MA 01760
/*                     All Rights Reserved
*/

/* This module ATTEMPTS to flush all dirty pages to disk and return
   the time it took to do this.  You may have to modify this for
   your specific system.

   Initial Version: 28 June 1990 by Barry Wolman

          Modified: 16 July 1990 by Barry Wolman
               1. Add unmount code and related subroutines

          Modified: 24 July 1990 by Barry Wolman
               1. Use ERR macros instead of gripe
               2. Change dounmounts to ignore errors

          Modified: 6 August 1990 by Barry Wolman
               1. Add several error checks

          Modified: 8 October 1990 by Barry Wolman
               1. Use mountpath & umountpath
               2. Fix bug in dounmounts

          Modified: 26 October 1990 by Barry Wolman
               1. Use SPEC standard mount_disks & unmount_disks
               2. Use ismounted() to determine whether or not to unmount
               3. Add check argument to dounmounts()
*/

#include <stdio.h>
#if UNIX
#include <fcntl.h>
#endif
#include "ioincl.h"

extern struct control control;
extern char *mountpath,*umountpath;

long ioflush(user,sys)
double *user,*sys;
    {
    long start,finish;
    int i,fid,errs = 0;
    double u1,s1,u2,s2;

    time(&start);

    if (control.mounts) {

         /* Unmount all mounted devices */

         fullcptimes(&u1,&s1);
         errs = dounmounts(0);
         fullcptimes(&u2,&s2);
         }
    else {

         cptimes(&u1,&s1);
#ifdef USE_FSYNC
         for(i = 0; i < control.n_files; i++) {
              if ((fid = open(control.path[i],O_RDWR)) < 0)
                   ERR("Could not reopen %s",control.path[i]);
              else {
                   if (fsync(fid) < 0) ERRS("Could not fsync %s",control.path[i]);
                   if (close(fid) < 0) ERRS("Could not close %s",control.path[i]);
                   }
              }
#else
#if UNIX
         sync();
         sync();
#endif
#endif
         cptimes(&u2,&s2);
         }

    *user = u2-u1;
    *sys = s2-s1;
    time(&finish);
    return(errs ? -1 : finish-start);
    }

/* This routine mounts the directories on the devices specified by the user.
   Exit is called (from ERRS) in the event of an error */

void domounts()
    {
    char mountline[256];
    register int i;

    for (i = 0; i < control.mounts; i++) {

         /* Note: argument sequence for mount_disks is DIR DEVICE,
                  /etc/mount command is DEVICE DIR */

         sprintf(mountline,"%s %s %s",mountpath,control.dir[i],
          control.device[i]);
         if (system(mountline)) ERRS("Could not system '%s'",mountline);
         }

    }

/* This routine unmounts all directories.
   Return of 1 indicates failure, return of 0 indictes success */

dounmounts(check)
int check;
    {
    char umountline[256];
    register int i;

    for(i = 0; i < control.mounts; i++) {
         if (!check || ismounted(control.device[i])) {	
              sprintf(umountline,"%s %s",umountpath,control.device[i]);
              if (system(umountline)) {
                   MSGS("Could not system '%s'",umountline);
                  return(1);
                  }
              }
         }

    return(0);
    }
