/* iosubs.c
/* Copyright (c) 1988, Prime Computer, Inc., Natick, MA 01760
/*                     All Rights Reserved
*/

/* Utility subroutines for IO benchmark.

   Initial Version: 12 August 1988 by Barry Wolman

          Modified: 29 August 1988 by Barry Wolman
              1. Add work subroutine
              2. Change parse_control to pick up work param

          Modified: 10 September 1988 by Barry Wolman
              1. Return error code when too few files specified
              2. Add indirect specification of files

          Modified:  2 October 1988 by Barry Wolman
              1. Set calibrate if n_mins is negative

          Modified:  3 December 1988 by Barry Wolman
              1. Increase checking of control file parameters
              2. Add rebuild_message subroutine

          Modified: 15 February 1989 by Tom Olson
              1. Add n_reads & n_writes parameters

          Modified: 22 February 1989 by Barry Wolman
              1. Cleanup error handling
              2. Still more error checking on parameters
              3. Rename parse_control to read_control and remove
                 indirection
              4. Move check_params and support subroutines into iomain

          Modified: 25 May 1989 by Barry Wolman
              1. Add variant to control

          Modified: 31 May 1989 by Barry Wolman
              1. Add server_request and gripe

          Modified:  3 September 1989 by Barry Wolman
              1. Correct #ifdef's in random
              2. Conditionally compile server code

          Modified: 28 June 1990 by Barry Wolman
              1. All times in seconds

          Modified: 24 July 1990 by Barry Wolman
              1. Rewrite gripe
              2. Use ERR macro instead of direct calls to gripe
              3. Error check lseek calls
              4. Add setcleanup

          Modified: 6 August 1990 by Barry Wolman
              1. Add more error checks

          Modified: 10 August 1990 by Barry Wolman
              1. Fix definition of malloc() to satisfy ANSI compilers

          Modified: 17 August 1990 by Barry Wolman
              1. Fix gripe for PRIMOS

          Modified: 8 October 1990 by Barry Wolman
              1. Delete obsolete unused random() code

          Modified: 16 October 1990 by Barry Wolman
              1. Use exit(1)
*/

#include <stdio.h>
#include "ioincl.h"
#include "ioserver.h"

#define TEMPLEN    256

void *malloc();
char *copystr();

#if UNIX
extern char *sys_errlist[];
#endif

/* Open and read the specified control file for the specified caller.  */

read_control(caller,cfname,cp)
char *caller, *cfname;
register struct control *cp;
    {
    register FILE *cf;
    register int errs,i;
    char temp[TEMPLEN];

    if ((cf = fopen(cfname,"r")) == NULL) ERRS("Could not open control file %s",cfname);

    errs = 0;
    if (getint(cf,&(cp -> runid))) errs++;
    if (getint(cf,&(cp -> runtype))) errs++;
    if (getint(cf,&(cp -> n_files))) errs++;
    if (getint(cf,&(cp -> n_recs))) errs++;
    if (getint(cf,&(cp -> reclen))) errs++;
    if (getint(cf,&(cp -> filelen))) errs++;
    if (getint(cf,&(cp -> n_users))) errs++;
    if (getint(cf,&(cp -> locks_per_file))) errs++;
    if (getint(cf,&(cp -> lock_sleep_time))) errs++;
    if (getint(cf,&(cp -> start_delay))) errs++;
    if (getint(cf,&(cp -> duration))) errs++;
    if (getint(cf,&(cp -> end_delay))) errs++;
    if (getint(cf,&(cp -> param))) errs++;
    if (getint(cf,&(cp -> n_reads))) errs++;
    if (getint(cf,&(cp -> n_writes))) errs++;
    if (getint(cf,&(cp -> trans))) errs++;
    if (getint(cf,&(cp -> variant))) errs++;

    /* There shouldn't be any errors since the file just read was
       created by iomain */

    if (errs) {
         MSG("Improper control file format");
         if (fclose(cf)) ERRS("Could not close %s",cfname);
         return(1);
         }

    if (cp -> duration >= 0) cp -> calibrate = 0;
    else {
         cp -> duration = - cp -> duration;
         cp -> calibrate = 1;
         }

    for(i=0;i < cp -> n_files; i++) {
         if (fgets(temp,TEMPLEN,cf) == NULL) {
              MSGSS("Only %d of required %d files specified",i,cp -> n_files);
              if (fclose(cf)) ERRS("Could not close %s",cfname);
              return(1);
              }

         temp[strlen(temp)-1] = 0;          /* Kill NL at end of pathname */
         cp -> path[i] = copystr(temp);
         }

    if (fclose(cf)) ERRS("Could not close %s",cfname);
    return(0);
    }

/* Get value of positive integer stored in first field of next line of
   specified file.  Return 1 if at end of file or if line is in wrong
   format or if integer is negative;  otherwise return 0 */

getint(fid,ip)
FILE *fid;
int *ip;
    {
    char line[256];

    if (fgets(line,256,fid) == NULL) return(1);
    if (sscanf(line,"%d ",ip) != 1) return(1);
    if (*ip < 0) return(1);
    return(0);
    }

/* Copy a string */

char *copystr(tp)
register char *tp;
    {
    register char *q;

    if ((q = (char *)malloc(strlen(tp)+1)) == NULL)
         ERRS("Could not malloc(%d)",strlen(tp)+1);

    strcpy(q,tp);
    return(q);
    }

/* Get record from specified file unit */

getrec(unitnum,recnum,recp,reclen)
int unitnum,recnum;
struct rec *recp;
    {
    if (lseek(unitnum,recnum * reclen,0) < 0)
         ERRS("Could not lseek (r) to record %d",recnum);
    if (read(unitnum,recp,reclen) < 0)
         ERRS("Could not read record %d",recnum);
    }

/* Put record to specified file unit */

putrec(unitnum,recnum,recp,reclen)
int unitnum,recnum;
struct rec *recp;
    {
    if (lseek(unitnum,recnum * reclen,0) < 0)
         ERRS("Could not lseek (w) to record %d",recnum);
    if (write(unitnum,recp,reclen) < 0)
         ERRS("Could not write record %d",recnum);
    }

char *ctime();

/* Slightly more convenient interface to ctime.  Trunc arg of 0
   means just drop NL at end.  Trunc arg non-zero means drop year */

char *cvtime(secs,trunc)
long secs;
    {
    register char *p;

    p = ctime(&secs);
    p[trunc ? 19 : strlen(p)-1] = 0;
    return(p);
    }

/* Burn some CPU time by looping */

work(param)
int param;
    {
    register int n, j, count;
    j = 1;
    count = 1000 * param;
    for(n = 0; n < count;n++);
       param = param + j;
    }

/* Send a message and wait for reply.  Value returned by server_request is
   rmsg.mitem[0] */

server_request(r)
int r;
    {
#ifdef MESSAGEQUEUES
    if (!serverpid) ERR("cannot send message, ioserver not running");

    smsg.mtype = r;
    smsg.mitem[0] = mypid;

    if (SEND(smsgid,smsg) < 0) ERR("msgsnd failed");

    /* Wait for an acknowledgment */

    if (RECEIVE(rmsgid,rmsg,mypid) < 0) ERR("msgrcv failed");

    return(rmsg.mitem[0]);
#else
    return(0);             /* Will never be called */
    }
#endif

/* Return name of current process */

char *procname(n)
int n;
    {
    static char temp[16];

    switch(n) {
         case -1:
              return("ioserver");

         case 0:
              return("iobench");

         default:
              sprintf(temp,"user %d",n);
              return(temp);
              }
      }


/* Remember cleanup procedure to be called by gripe */

static
#if PRIMOS
fortran
#endif
int (*cleanup_proc)() = NULL;

setcleanup(proc)
int (*proc)();
    {
    cleanup_proc = proc;
    }

/* Print error (annotated if UNIX) and exit */

#if UNIX
#define ERRINFO (sys_errlist[errno])
#endif

#if PRIMOS
#define ERRINFO "No Information Available"
#endif

gripe(str,stop)
char *str;
int stop;
    {

#if UNIX
    if (errno) fprintf(stderr,"%s: %s - %s\n",procname(user),str,ERRINFO);
    else fprintf(stderr,"%s: %s\n",procname(user),str);
#endif

#if PRIMOS
    char buff[256];

    if (errno) {
         sprintf(buff,"%s: %s",procname(user),str);
         perror(buff);
         }
    else fprintf(stderr,"%s: %s\n",procname(user),str);
#endif

    if (stop) {
         if (cleanup_proc) (*cleanup_proc)(0);
         exit(1);
         }
    }
